'use strict';

var _interopRequireWildcard = require('babel-runtime/helpers/interop-require-wildcard')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.ar = ar;
exports.plural = plural;
exports.varar = varar;
exports.aropt = aropt;
exports.toArrayBuffer = toArrayBuffer;
exports.fromCamelCase = fromCamelCase;
exports.toCamelCase = toCamelCase;
exports.recursivelyConvertPseudotype = recursivelyConvertPseudotype;
exports.mkAtom = mkAtom;
exports.mkSeq = mkSeq;
exports.mkErr = mkErr;

var _errors = require('./errors');

var err = _interopRequireWildcard(_errors);

var _base = require('./base');

function ar(fn) {
  return function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    if (args.length !== fn.length) {
      throw new err.RqlDriverError('Expected ' + fn.length + ' argument' + plural(fn.length) + ' but found ' + args.length + '.');
    }
    return fn.apply(this, args);
  };
}

function plural(num) {
  return num == 1 ? '' : 's';
}

function varar(min, max, fn) {
  return function () {
    for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    if ((0, _base.exists)(min) && args.length < min || (0, _base.exists)(max) && args.length > max) {
      if ((0, _base.exists)(min) && !(0, _base.exists)(max)) {
        throw new err.RqlDriverError('Expected ' + min + ' or more arguments but found ' + args.length + '.');
      }
      if ((0, _base.exists)(max) && !(0, _base.exists)(min)) {
        throw new err.RqlDriverError('Expeted ' + max + ' or fewer arguments but found ' + args.length + '.');
      }
      throw new err.RqlDriverError('Expected between ' + min + ' and ' + max + ' arguments but found ' + args.length + '.');
    }
    return fn.apply(this, args);
  };
}

function aropt(fn) {
  return function () {
    var expectedPosArgs = fn.length - 1;

    for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      args[_key3] = arguments[_key3];
    }

    var perhapsOptDict = args[expectedPosArgs];
    if ((0, _base.exists)(perhapsOptDict) && Object.prototype.toString.call(perhapsOptDict) !== '[object Object]') {
      perhapsOptDict = null;
    }

    var numPosArgs = args.length - (perhapsOptDict ? 1 : 0);

    if (expectedPosArgs !== numPosArgs) {
      throw new err.RqlDriverError('Expected ' + expectedPosArgs + ' argument' + plural(expectedPosArgs) + ' (not including options) but found ' + numPosArgs + '.');
    }
    return fn.apply(this, args);
  };
}

function toArrayBuffer(node_buffer) {
  var arr = new Uint8Array(new ArrayBuffer(node_buffer.length));
  for (var i = 0, len = node_buffer.length; i < len; i++) {
    arr[i] = node_buffer[i];
  }
  return arr.buffer;
}

function fromCamelCase(token) {
  return token.replace(/[A-Z]/g, function (match) {
    return '_' + match.toLowerCase();
  });
}

function toCamelCase(token) {
  return token.replace(/_[a-z]/g, function (match) {
    return match[1].toUpperCase();
  });
}

function convertPseudotype(obj, opts) {
  switch (obj['$reql_type$']) {
    case 'TIME':
      switch (opts.timeFormat) {
        case 'native':
        case undefined:
          if (!(0, _base.exists)(obj['epoch_time'])) {
            throw new err.RqlDriverError('psudo-type TIME ' + obj + ' object missing expected field \'epoch_time\'.');
          }
          return new Date(obj['epoch_time'] * 1000);
        case 'raw':
          return obj;
        default:
          throw new err.RqlDriverError('Unknown timeFormat run option ' + opts.timeFormat + '.');
      }
    case 'GROUPED_DATA':
      switch (opts.groupFormat) {
        case 'native':
        case undefined:
          var res = [];
          for (var i = 0, len = obj['data'].length; i < len; i++) {
            res.push({
              group: i[0],
              reduction: i[1]
            });
          }
          return res;
        case 'raw':
          return obj;
        default:
          throw new err.RqlDriverError('Unknown groupFormat run option ' + opts.groupFormat + '.');
      }
    case 'BINARY':
      switch (opts.binaryFormat) {
        case 'native':
        case undefined:
          if (!(0, _base.exists)(obj['data'])) {
            throw new err.RqlDriverError('pseudo-type BINARY object missing expected field \'data\'.');
          }
          return new Buffer(obj['data'], 'base64');
        case 'raw':
          return obj;
        default:
          throw new err.RqlDriverError('Unknown binaryFormat run option ' + opts.binaryFormat + '.');
      }
  }
}

function recursivelyConvertPseudotype(obj, opts) {
  if (Array.isArray(obj)) {
    for (var i = 0, len = obj.length; i < len; i++) {
      obj[i] = recursivelyConvertPseudotype(obj[i], opts);
    }
  } else if (obj && typeof obj === 'object') {
    for (var key in obj) {
      obj[key] = recursivelyConvertPseudotype(obj[key], opts);
      obj = convertPseudotype(obj, opts);
    }
  }
  return obj;
}

function mkAtom(response, opts) {
  return recursivelyConvertPseudotype(response.r[0], opts);
}

function mkSeq(response, opts) {
  return recursivelyConvertPseudotype(response.r, opts);
}

function mkErr(ErrClass, response, root) {
  return new ErrClass(mkAtom(response), root, response.b);
}
//# sourceMappingURL=util.js.map